/*************************************************************************/
/***                                                                   ***/
/**                              Algo2.c                                **/
/***                                                                   ***/
/*************************************************************************/
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include <dos.h>
#include <errno.h>
#include "ctl3d.h"
#include "algo2.h"

/*************************************************************************/
/***                                                                   ***/
/**                      DoClassRegistration                            **/
/***                                                                   ***/
/*************************************************************************/
BOOL DoClassRegistration( HINSTANCE hInstance )
{
	WNDCLASS   wndclass;
	memset( &wndclass, 0x00, sizeof( WNDCLASS ) );                           
	wndclass.style = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc = MainWindowProc;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;
	wndclass.hInstance = hInstance;
	wndclass.hIcon = LoadIcon( NULL, IDI_APPLICATION );
	wndclass.hCursor = LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground = ( HBRUSH )( COLOR_WINDOW + 1 );
	wndclass.lpszMenuName = MAKEINTRESOURCE( IDR_MENU );
	wndclass.lpszClassName = "AlgoSample";
	return RegisterClass( &wndclass );
}
           
/*************************************************************************/
/***                                                                   ***/
/**                           InitInstance                              **/
/***                                                                   ***/
/*************************************************************************/
BOOL InitInstance( HANDLE hInstance, int nCmdShow )
{
    HWND hWnd;
    hAppInst = hInstance;   
    Ctl3dRegister( hInstance );
    hWnd = CreateWindow( "AlgoSample", "Algorithm Sample Application",
						WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT,
						CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL );
    if( !hWnd ) 
	{	
		Ctl3dUnregister( hInstance );
		return 0;
	}
    ShowWindow( hWnd, nCmdShow );
    UpdateWindow( hWnd );
    return 1;
}

/*************************************************************************/
/***                                                                   ***/
/**                             WinMain                                 **/
/***                                                                   ***/
/*************************************************************************/
int FAR PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow )
{
    MSG Message;

    if( !hPrevInstance && !DoClassRegistration( hInstance ) )
        return 0;
    if( !InitInstance( hInstance, nCmdShow ) )
        return 0;
    while( GetMessage( &Message, NULL, 0, 0 ) )
    {
        TranslateMessage( &Message );
        DispatchMessage( &Message );
    }                          
    Ctl3dUnregister( hInstance );
    return Message.wParam;
}
 
/*************************************************************************/
/***                                                                   ***/
/**                          MainWindowProc                             **/
/***                                                                   ***/
/*************************************************************************/
long FAR PASCAL __export MainWindowProc( HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam )
{
	HDC hDC;
	PAINTSTRUCT ps;     
	HMENU hMenu;
	
	switch ( Message ) 
	{
		case WM_CREATE:
			Ctl3dAutoSubclass( hAppInst );
			break;
		case WM_PAINT: 
			hDC = BeginPaint( hWnd, &ps );
			SetBkMode( hDC, TRANSPARENT );
			EndPaint( hWnd, &ps );
			break;
		case WM_SYSCOLORCHANGE:
			Ctl3dColorChange();
			break;                  
		case WM_COMMAND:
			hMenu = GetMenu( hWnd );
			switch ( wParam ) 
			{
				case ID_CALCULATE:
				{	
					int i;
					DLGPROC lpProc = ( DLGPROC ) MakeProcInstance( ( FARPROC ) DialogProc, hAppInst );
					i = DialogBox( hAppInst, "Calculator", hWnd, lpProc );
					FreeProcInstance( lpProc );          
					if( i )
						CalculateExpression( hWnd );
					return i;
				}
			}
            break;
		case WM_DESTROY :
			PostQuitMessage( 0 );
			return 0;
		default:
			return DefWindowProc( hWnd, Message, wParam, lParam );
	}
	return 0;
}              

/*************************************************************************/
/***                                                                   ***/
/**                            CenterWindow                             **/
/***                                                                   ***/
/*************************************************************************/
int CenterWindow( HWND hWnd, int Show ) 
{
	RECT ParentRect, ChildRect;
    HWND hParentWnd = GetParent( hWnd );
	int x, y, Width, Height;                                     
	GetWindowRect( hParentWnd, &ParentRect );
	GetWindowRect( hWnd, &ChildRect );
	Width = ( ChildRect.right - ChildRect.left ) + 1;
	Height = ( ChildRect.bottom - ChildRect.top ) + 1;
	x = ( ParentRect.right - ParentRect.left + 1 - Width ) / 2;
	y = ( ParentRect.bottom - ParentRect.top + 1 - Height ) / 2;
	MoveWindow( hWnd, ( x < 0 ? 0 : x ), ( y < 0 ? 0 : y ), Width, Height, Show );
	return TRUE;
}

/*************************************************************************/
/***                                                                   ***/
/**                             DialogProc                              **/
/***                                                                   ***/
/*************************************************************************/
LRESULT _export  FAR PASCAL	DialogProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	PAINTSTRUCT ps;
		
	switch( msg )
	{
		case WM_INITDIALOG:
			Parentheses = 0;
			Head = NULL;
			CenterWindow( hDlg, TRUE );
			return 0;
		case WM_PAINT:
			BeginPaint( hDlg, &ps );
			EndPaint( hDlg, &ps );
			return 0;
		case WM_SYSCOLORCHANGE:
			Ctl3dColorChange();
			break;
		case WM_COMMAND:
		{
			switch( wParam ) 
			{
				case IDOK:
					GetDlgItemText( hDlg, IDC_EXPRESSION, ( LPSTR ) Expression, sizeof( Expression ) );
					EndDialog( hDlg, TRUE );
					break;
				case IDCANCEL: 
					EndDialog( hDlg, FALSE );
					break;
			}
			break;
		}
	}
	return 0;                                
}

/*************************************************************************/
/***                                                                   ***/
/**                        CalculateExpression                          **/
/***                                                                   ***/
/*************************************************************************/
void CalculateExpression( HWND hWnd )
{
	int Result;
	if( !ConvertToPostfix() )
	{
		while( Head != NULL )
			Pop( &Result );	
		MessageBox( hWnd, "Izraz nije adekvatno unet!", "Greska", MB_OK );	
		return;
	}		
	Result = CalculateResult();
    sprintf( Expression, "Izraz ima vrednost: %d.", Result );
	MessageBox( hWnd, Expression, "Rezultat", MB_OK );	
}   

/*************************************************************************/
/***                                                                   ***/
/**                                Push                                 **/
/***                                                                   ***/
/*************************************************************************/
BOOL Push( int Number )
{
	LPELEMENT New = ( LPELEMENT ) malloc( sizeof( ELEMENT ) );
	if( !New ) return FALSE;
	New->Value = Number; New->Next = Head;
	Head = New;
	return TRUE;
}

/*************************************************************************/
/***                                                                   ***/
/**                                 Pop                                 **/
/***                                                                   ***/
/*************************************************************************/
BOOL Pop( int* Number )
{
	LPELEMENT Element = Head;
	if( !Head ) return FALSE; 
	*Number = Head->Value;          
	Head = Head->Next;
	free( Element );
	return TRUE;
}

/*************************************************************************/
/***                                                                   ***/
/**                         ConvertToPostfix                            **/
/***                                                                   ***/
/*************************************************************************/
BOOL ConvertToPostfix()
{
	char String[MAX_BUF_LEN+1];
	int i = 0, j = 0, Operation;
	while( ( i < MAX_BUF_LEN ) && ( Expression[i] != '\0' ) )
	{
		if( Expression[i] == ')' ) 
		{	
			if( --Parentheses < 0 ) return FALSE;
			Pop( &Operation );
			j += sprintf( String + j, "%1c", ( char ) Operation );
		}
		if( Expression[i] == '+' )
			Push( ( int ) Expression[i] );
		if( Expression[i] == '*' )
			Push( ( int ) Expression[i] );
		if( Expression[i] == '-' )
			Push( ( int ) Expression[i] );
		if( Expression[i] == '/' )
			Push( ( int ) Expression[i] );
		if( Expression[i] >= '0' && Expression[i] <= '9' ) 
		{
			if( i == 0 ) return FALSE;
			while( Expression[i] >= '0' && Expression[i] <= '9' ) 
			{
				if( j == MAX_BUF_LEN ) return FALSE; 
				j += sprintf( String + j, "%1c", Expression[i] );
				i++;
			}
			j += sprintf( String + j, " " );               
			i--;
		}
		if( Expression[i] == '(' ) 
			Parentheses++;
		if( j == MAX_BUF_LEN ) return FALSE; 
		i++;
	}
	if( Parentheses > 0 ) return FALSE;
	strcpy( Expression, String );
	return TRUE;
}

/*************************************************************************/
/***                                                                   ***/
/**                          CalculateResult                            **/
/***                                                                   ***/
/*************************************************************************/
int CalculateResult()
{    
	int Result, FirstOperand, SecondOperand, i = 0;
	while( ( i <= MAX_BUF_LEN ) && ( Expression[i] != '\0' ) )
	{
		Result = 0;
		if( Expression[i] == '+' )
		{
			Pop( &SecondOperand );
			Pop( &FirstOperand );
			Result = FirstOperand + SecondOperand;
		}
		if( Expression[i] == '*' )
		{
			Pop( &SecondOperand );
			Pop( &FirstOperand );
			Result = FirstOperand * SecondOperand;
		}
		if( Expression[i] == '-' )
		{
			Pop( &SecondOperand );
			Pop( &FirstOperand );
			Result = FirstOperand - SecondOperand;
		}
		if( Expression[i] == '/' )
		{
			Pop( &SecondOperand );
			Pop( &FirstOperand );
			Result = FirstOperand / SecondOperand;
		}
		while( Expression[i] >= '0' && Expression[i] <= '9' )
		{
			Result = 10 * Result + ( Expression[i] - '0');
			i++;
		}
		Push( Result );                           
		i++;
	}	
	return Result;
}





